from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from ..form import GetFormMixin


class CoursePaymentsForm(GetFormMixin, forms.Form):
    date_start  = forms.DateField(label=_('Start date'))
    date_end    = forms.DateField(label=_('End date'))


class CoursePaymentsStatusForm(GetFormMixin, forms.Form):
    date    = forms.DateField(label=_('Status for date'))


class CourseStatsForm(GetFormMixin, forms.Form):
    date        = forms.DateField(label=_('Statistics for date'))
    paid_only   = forms.BooleanField(label=_('Count only paid registrations'), required=False)



from __future__ import unicode_literals

from django.contrib.auth.decorators import user_passes_test
from django.core.urlresolvers import reverse_lazy as reverse
from django.views.decorators.csrf import csrf_exempt

from . import (
    courses, leaders, messages, parents, participants, registrations, reports,
    schoolyear, subjects, summaries, terms_conditions, timesheets, user,
)
from ..conf import settings
from .reports import (
    courses as reports_courses, debtors as reports_debtors,
    events as reports_events,
)

login_url = reverse('leprikon:user_login')


def _user_is_leader(u):
    try:
        u.leprikon_leader
        return True
    except:
        return False


login_required = user_passes_test(
    lambda u: u.is_authenticated(),
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)

leader_required = user_passes_test(
    _user_is_leader,
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)

leader_or_staff_required = user_passes_test(
    lambda u: u.is_staff or _user_is_leader(u),
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)

staff_required = user_passes_test(
    lambda u: u.is_staff,
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)


summary                         = login_required(summaries.SummaryView.as_view())

user_create                     = user.UserCreateView.as_view()
user_update                     = login_required(user.UserUpdateView.as_view())
user_login                      = user.user_login
user_logout                     = user.user_logout
user_password                   = user.user_password
password_reset                  = user.password_reset
password_reset_done             = user.password_reset_done
password_reset_confirm          = user.password_reset_confirm
password_reset_complete         = user.password_reset_complete

registration_list               = login_required(registrations.RegistrationsListView.as_view())
registration_pdf                = login_required(subjects.SubjectRegistrationPdfView.as_view())
registration_cancel             = login_required(subjects.SubjectRegistrationCancelView.as_view())

payment_list                    = login_required(subjects.SubjectPaymentsListView.as_view())
payment_pdf                     = login_required(subjects.SubjectPaymentPdfView.as_view())

participant_list                = login_required(participants.ParticipantListView.as_view())
participant_create              = login_required(participants.ParticipantCreateView.as_view())
participant_update              = login_required(participants.ParticipantUpdateView.as_view())

parent_create                   = login_required(parents.ParentCreateView.as_view())
parent_update                   = login_required(parents.ParentUpdateView.as_view())

leader_summary                  = leader_required(summaries.LeaderSummaryView.as_view())
course_alternating              = leader_required(courses.CourseAlternatingView.as_view())
course_journal                  = leader_or_staff_required(courses.CourseJournalView.as_view())
coursejournalentry_create       = leader_or_staff_required(courses.CourseJournalEntryCreateView.as_view())
coursejournalentry_update       = leader_or_staff_required(courses.CourseJournalEntryUpdateView.as_view())
coursejournalentry_delete       = leader_or_staff_required(courses.CourseJournalEntryDeleteView.as_view())
coursejournalleaderentry_update = leader_or_staff_required(courses.CourseJournalLeaderEntryUpdateView.as_view())
coursejournalleaderentry_delete = leader_or_staff_required(courses.CourseJournalLeaderEntryDeleteView.as_view())

subject_list                    = subjects.SubjectListView.as_view()
subject_list_mine               = leader_required(subjects.SubjectListMineView.as_view())
subject_detail                  = subjects.SubjectDetailView.as_view()
subject_registration_form       = login_required(subjects.SubjectRegistrationFormView.as_view())
subject_update                  = leader_or_staff_required(subjects.SubjectUpdateView.as_view())
subject_registrations           = leader_or_staff_required(subjects.SubjectRegistrationsView.as_view())

message_list                    = login_required(messages.MessageListView.as_view())
message_detail                  = csrf_exempt(messages.MessageDetailView.as_view())

leader_list                     = leaders.LeaderListView.as_view()

timesheet_list                  = leader_required(timesheets.TimesheetListView.as_view())
timesheet_detail                = leader_required(timesheets.TimesheetDetailView.as_view())
timesheet_submit                = leader_required(timesheets.TimesheetSubmitView.as_view())
timesheetentry_create           = leader_required(timesheets.TimesheetEntryCreateView.as_view())
timesheetentry_update           = leader_required(timesheets.TimesheetEntryUpdateView.as_view())
timesheetentry_delete           = leader_required(timesheets.TimesheetEntryDeleteView.as_view())

school_year                     = schoolyear.SchoolYearView.as_view()

report_list                     = staff_required(reports.ReportsListView.as_view())
report_course_payments          = staff_required(reports_courses.ReportCoursePaymentsView.as_view())
report_course_payments_status   = staff_required(reports_courses.ReportCoursePaymentsStatusView.as_view())
report_course_stats             = staff_required(reports_courses.ReportCourseStatsView.as_view())
report_event_payments           = staff_required(reports_events.ReportEventPaymentsView.as_view())
report_event_payments_status    = staff_required(reports_events.ReportEventPaymentsStatusView.as_view())
report_event_stats              = staff_required(reports_events.ReportEventStatsView.as_view())
report_debtors                  = staff_required(reports_debtors.ReportDebtorsView.as_view())

terms_conditions                = terms_conditions.TermsConditionsView.as_view()



from __future__ import unicode_literals

from collections import namedtuple

from django.core.urlresolvers import reverse_lazy as reverse
from django.db.models import Sum
from django.template.response import TemplateResponse
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from . import ReportBaseView
from ...conf import settings
from ...forms.reports.events import (
    EventPaymentsForm, EventPaymentsStatusForm, EventStatsForm,
)
from ...models.agegroup import AgeGroup
from ...models.events import Event, EventRegistration
from ...models.roles import Participant
from ...models.subjects import SubjectPayment, SubjectType


class ReportEventPaymentsView(ReportBaseView):
    form_class      = EventPaymentsForm
    template_name   = 'leprikon/reports/event_payments.html'
    title           = _('Event payments')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['payments'] = SubjectPayment.objects.filter(
            registration__subject__subject_type__subject_type=SubjectType.EVENT,
            created__gte=context['date_start'],
            created__lte=context['date_end'],
        )
        context['sum'] = context['payments'].aggregate(Sum('amount'))['amount__sum']
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



class ReportEventPaymentsStatusView(ReportBaseView):
    form_class      = EventPaymentsStatusForm
    template_name   = 'leprikon/reports/event_payments_status.html'
    title           = _('Event payments status')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    EventPaymentsStatusSums = namedtuple('EventPaymentsStatusSums', ('registrations', 'status'))

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['reports'] = [
            self.Report(event, context['date'])
            for event in Event.objects.filter(school_year=self.request.school_year)
        ]
        context['sum'] = self.EventPaymentsStatusSums(
            registrations   = sum(len(r.registrations)  for r in context['reports']),
            status          = sum(r.status              for r in context['reports']),
        )
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))

    class Report:
        def __init__(self, event, d):
            self.event = event
            self.date = d

        @cached_property
        def registrations(self):
            return list(self.event.registrations.filter(
                approved__lte=self.date,
            ))

        RegPaymentStatus = namedtuple('RegPaymentStatus', ('registration', 'status'))

        @cached_property
        def registration_statuses(self):
            return [
                registration_status for registration_status in (
                    self.RegPaymentStatus(
                        registration = registration,
                        status       = registration.eventregistration.get_payment_status(self.date),
                    )
                    for registration in self.registrations
                ) if registration_status.status.receivable
            ]

        @cached_property
        def status(self):
            return sum(rs.status for rs in self.registration_statuses)



class ReportEventStatsView(ReportBaseView):
    form_class      = EventStatsForm
    template_name   = 'leprikon/reports/event_stats.html'
    title           = _('Event statistics')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    ReportItem      = namedtuple('ReportItem', ('age_group', 'all', 'boys', 'girls', 'local', 'eu', 'noneu'))

    all_EU_countries    = [
        'AT', 'BE', 'BG', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE', 'IT',
        'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE', 'GB',
    ]
    other_EU_countries = [country for country in all_EU_countries if country != settings.LEPRIKON_COUNTRY]

    def form_valid(self, form):
        d               = form.cleaned_data['date']
        paid_only       = form.cleaned_data['paid_only']
        context         = form.cleaned_data
        context['form'] = form

        events = Event.objects.filter(school_year=self.request.school_year)
        context['events_count'] = events.count()

        registrations = EventRegistration.objects.filter(subject__in=events, approved__lte=d).exclude(canceled__lte=d)
        if paid_only:
            registrations = [
                reg for reg in registrations
                if reg.get_payment_status(d).balance >= 0
            ]
        else:
            registrations = list(registrations)

        context['registrations_counts'] = self.ReportItem(
            age_group=None,
            all=len(registrations),
            boys=len([r for r in registrations if r.participant_gender == Participant.MALE]),
            girls=len([r for r in registrations if r.participant_gender == Participant.FEMALE]),
            local=len([r for r in registrations if r.participant_citizenship == settings.LEPRIKON_COUNTRY]),
            eu=len([r for r in registrations if r.participant_citizenship in self.other_EU_countries]),
            noneu=len([r for r in registrations if r.participant_citizenship not in self.all_EU_countries]),
        )
        context['registrations_counts_by_age_groups'] = []
        for age_group in AgeGroup.objects.all():
            regs = [r for r in registrations if r.participant_age_group == age_group]
            context['registrations_counts_by_age_groups'].append(self.ReportItem(
                age_group=age_group,
                all=len(regs),
                boys=len([r for r in regs if r.participant_gender == Participant.MALE]),
                girls=len([r for r in regs if r.participant_gender == Participant.FEMALE]),
                local=len([r for r in regs if r.participant_citizenship == settings.LEPRIKON_COUNTRY]),
                eu=len([r for r in regs if r.participant_citizenship in self.other_EU_countries]),
                noneu=len([r for r in regs if r.participant_citizenship not in self.all_EU_countries]),
            ))

        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from ..form import GetFormMixin


class DebtorsForm(GetFormMixin, forms.Form):
    date = forms.DateField(label=_('Status for date'))
